unit LPrinter;

{
     Cytizen CT-S2000
      
<> -     Convert()
[] -      
}

interface
uses
{$IFDEF DELPHI}
windows,
{$ELSE}
LConvEncoding,
{$ENDIF}
Classes, DataPort, SysUtils, Graphics, ExtCtrls;

type

  { TLinePrinter }

  TLinePrinter = class(TComponent)
  private
    FStatusMode: Byte;
    FPrinterStatus: Byte;
    FOfflineStatus: Byte;
    FErrorStatus: Byte;
    FOnStatus: TGetStrProc;
    FDataPort: TDataPort;
    FLastStatusStr: string;
    FReadyToPrint: Boolean;
    FStatusTimer: TTimer;
    procedure OnDataAppearHandler(Sender: TObject);
    procedure FSetDataPort(Value: TDataPort);
    procedure OnStatusTimerHandler(Sender: TObject);
  protected
    FTemplate: string;
    //     ,   -  ,     -  .
    //    
    function LoadTextImage(fname: string): string;
    //     .
    //    
    function LoadBitmapImage(fname: string): string;
    //      ,   
    function Convert(text: AnsiString; params: TStrings): AnsiString;
    //      ,   
    //  ,     
    function ConvertSimple(text: AnsiString; params: TStrings): AnsiString;
  public
    RawDocument: AnsiString;
    Params: TStringList;
    Sections: TStringList;
    SimpleMode: boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetParam(AName, AValue: string);
    //    ,    
    function LoadTemplate(FileName: string; SectionName: string = ''): Boolean;
    //   
    procedure ClearDoc();
    //    .      
    procedure AddSection(ASectionName: string = ''; AParams: TStrings = nil);
    //    
    function Print(): Boolean;
    //    
    function PrintToFile(AFileName: string): Boolean;
    //   
    function CheckStatus(AStatusMode: Byte = 1): Boolean;
    //     
    function ReadyToPrint(): Boolean;
  published
    //    
    property Template: string read FTemplate write FTemplate;
    // DataPort 
    property DataPort: TDataPort read FDataPort write FSetDataPort;
    //     ,     
    property OnStatus: TGetStrProc read FOnStatus write FOnStatus;
  end;

//    ,       
function StrAlignR(s: string; len: Integer): string;
//    ,       
function StrAlignL(s: string; len: Integer): string;
//    ,       
function Utf8StrAlignR(s: string; len: Integer): string;
//    ,       
function Utf8StrAlignL(s: string; len: Integer): string;

procedure Register;

implementation

function StrAlignR(s: string; len: Integer): string;
var
  i: integer;
begin
  Result:='';
  for i:=1 to len-Length(s) do
  begin
    Result:=Result+' ';
  end;
  Result:=Result+s;
end;

function StrAlignL(s: string; len: Integer): string;
var
  i: integer;
begin
  Result:=s;
  for i:=1 to len-Length(s) do
  begin
    Result:=Result+' ';
  end;
end;

function Utf8StrAlignR(s: string; len: Integer): string;
var
  ss: string;
begin
  ss:=UTF8ToCP1251(s);
  Result:=CP1251ToUTF8(StrAlignR(ss, len));
end;

function Utf8StrAlignL(s: string; len: Integer): string;
var
  ss: string;
begin
  ss:=UTF8ToCP1251(s);
  Result:=CP1251ToUTF8(StrAlignL(ss, len));
end;

procedure Register;
begin
  RegisterComponents('CarddexLib', [TLinePrinter]);
end;

{ TLinePrinter }

constructor TLinePrinter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Params:=TStringList.Create();
  Sections:=TStringList.Create();

  FLastStatusStr:='';
  FStatusMode:=$00;
  FPrinterStatus:=$00;
  FOfflineStatus:=$00;
  FErrorStatus:=$00;
  FReadyToPrint:=True;

  FStatusTimer:=TTimer.Create(self);
  FStatusTimer.Interval:=1000;
  FStatusTimer.OnTimer:=@OnStatusTimerHandler;
  FStatusTimer.Enabled:=True;
end;

destructor TLinePrinter.Destroy();
begin
  FStatusTimer.Enabled:=False;
  FStatusTimer.OnTimer:=nil;
  FreeAndNil(FStatusTimer);
  Sections.Free();
  Params.Free();
  inherited Destroy();
end;

{$ifdef DELPHI}
function ToAnotherCodePage(Source : String; FromCodePage, ToCodePage : LongWord) : string;
type
  byte_arr = array of byte;

var
  byte_buffer_wide : pbyte;
  byte_buffer : pbyte;
  l, i :integer;

begin
  l := Length(Source);
  GetMem(byte_buffer_wide, l*2);
  fillchar(byte_buffer_wide^, l*2, 0);

  MultiByteToWideChar(FromCodePage, 0,  PChar(Source), l, PWideChar(byte_buffer_wide), l*2);

  GetMem(byte_buffer, l);
  WideCharToMultiByte(ToCodePage, 0,  PWideChar(byte_buffer_wide), l, PChar(byte_buffer), l, nil, nil);

  Result := copy(string(byte_buffer), 1, l);

  FreeMem(byte_buffer_wide, l*2);
  FreeMem(byte_buffer, l);
end;
{$endif}

// -----------------------------------------------------------------------------
function StrToHex( S : String) : String;
var
  i: Integer;
begin
  Result:=EmptyStr;
  for i:=1 to Length(S) do
    Result := Result + IntToHex(Ord(S[i]), 2)+' ';
end;

procedure MyStringReplace(var sWhere: string; const sFrom, sTo: string);
var
  spos, len: Integer;
  s2: string;
begin
  // StringReplace()  !
  len:=Length(sFrom);
  spos:=Pos(sFrom, sWhere);
  while spos>0 do
  begin
    s2:=Copy(sWhere, 1, spos-1)+sTo+Copy(sWhere, spos+len, MaxInt);
    sWhere:=s2;
    spos:=Pos(sFrom, sWhere);
  end;
end;

function TLinePrinter.CheckStatus(AStatusMode: Byte = 1): Boolean;
begin
  Result:=False;
  if not Assigned(DataPort) then Exit;
  if not DataPort.Active then Exit;
  FStatusMode:=AStatusMode;
  if AStatusMode = $01 then
  begin
    FPrinterStatus:=$00;
    FOfflineStatus:=$00;
    FErrorStatus:=$00;
  end;
  //self.DataPort.Push(#$10+#$04+Chr(AStatusMode));
  Result:=True;
end;

function TLinePrinter.Convert(text: AnsiString; params: TStrings): AnsiString;
var
  s, s2, sf, st: string;
  ESC, CR, LF, FF, GS, TAB, CAN: string;
  sl: TStringList;
  i: Integer;

begin
  ESC:=Chr($1B);   //
  CR:=Chr($0D);    // Caret Return
  LF:=Chr($0A);    // Line Feed
  FF:=Chr($0C);    //
  GS:=Chr($1D);
  TAB:=Chr($09);   // Horizontal Tab
  CAN:=Chr($18);   // Cancel print
  sl:=TStringList.Create();
  sl.Values['LINE']:='________________________________________________'+LF;
  sl.Values['TAB']:=TAB;
  sl.Values['CR']:=CR;  // Caret Return
  sl.Values['LF']:=LF;  // Line Feed
  sl.Values['FF']:=FF;  // Form Feed
  sl.Values['RESET']:=ESC+'@';  // Initialize printer
  // Cut paper fully
  sl.Values['CUT_FULL']:=LF+LF+LF+ESC+'i';
  // Cut paper partially
  sl.Values['CUT_PART']:=LF+LF+LF+ESC+'m';
  // Font 0: 12x24, 1: 9x24, 2: 8x16
  //sl.Values['FONT']:=ESC+'M';
  sl.Values['FONT_0']:=ESC+'M'+#0;
  sl.Values['FONT_1']:=ESC+'M'+#1;
  sl.Values['FONT_2']:=ESC+'M'+#2;
  // 0: USA
  sl.Values['CHARSET']:=ESC+'R';
  // 0: PC437, 7: PC866
  //sl.Values['CODETABLE']:=ESC+'t';
  sl.Values['CODETABLE_DEF']:=ESC+'t'+#00;
  sl.Values['CODETABLE_866']:=ESC+'t'+#07;
  // HEX 'hv', h-horizontal, v-vertical multiplier
  // $00 - standard, $11 - double, $22 - triple, etc..
  //sl.Values['SIZE']:=GS+'!';
  sl.Values['SIZE_00']:=GS+'!'+Chr($00);
  sl.Values['SIZE_01']:=GS+'!'+Chr($01);
  sl.Values['SIZE_10']:=GS+'!'+Chr($10);
  sl.Values['SIZE_11']:=GS+'!'+Chr($11);
  sl.Values['SIZE_12']:=GS+'!'+Chr($12);
  sl.Values['SIZE_21']:=GS+'!'+Chr($21);
  sl.Values['SIZE_22']:=GS+'!'+Chr($22);
  // 0-none, 1-one dot, 2-two dots
  //sl.Values['UNDERLINE']:=ESC+'-';
  sl.Values['UNDERLINE_0']:=ESC+'-'+#0;
  sl.Values['UNDERLINE_1']:=ESC+'-'+#1;
  // 0 or 1
  //sl.Values['BOLD']:=ESC+'G';
  sl.Values['BOLD_0']:=ESC+'G'+#0;
  sl.Values['BOLD_1']:=ESC+'G'+#1;
  // 0 or 1
  //sl.Values['INVERT']:=GS+'B';
  sl.Values['INVERT_0']:=GS+'B'+#0;
  sl.Values['INVERT_1']:=GS+'B'+#1;
  // 0 or 1, for sizes > 1
  //sl.Values['SMOOTH']:=GS+'b';
  sl.Values['SMOOTH_0']:=GS+'b'+#0;
  sl.Values['SMOOTH_1']:=GS+'b'+#1;
  // 0: left, 1: center, 2: right
  //sl.Values['ALIGN']:=GS+'a';
  sl.Values['ALIGN_L']:=GS+'a'+#0;
  sl.Values['ALIGN_C']:=GS+'a'+#1;
  sl.Values['ALIGN_R']:=GS+'a'+#2;
  sl.Values['BEEP']:=ESC+'RS';    // Bufferized!
  //sl.Values['TEST']:=GS+'( A pL pH n m';
  // tab positions values, followed by $00
  sl.Values['SET_HT']:=ESC+'D';
  sl.Values['IMAGE_1']:=self.LoadTextImage('img1.tpl');
  sl.Values['PAGE_MODE']:=ESC+'L';  // PAGE mode
  sl.Values['STANDARD_MODE']:=ESC+'S';  // STANDARD mode

  s2:=text;
  {$ifdef DELPHI}
  s:=ToAnotherCodePage(s2, 1251, 866);
  {$else}
  s:=UTF8ToCP866(s2);
  {$endif}
  //  
  for i:=0 to sl.Count-1 do
  begin
    sf:='<'+sl.Names[i]+'>';
    st:=sl.ValueFromIndex[i];

    // StringReplace()  !
    MyStringReplace(s, sf, st);
    {len:=Length(sf);
    spos:=Pos(sf, s);
    while spos>0 do
    begin
      s2:=Copy(s, 1, spos-1)+st+Copy(s, spos+len, MaxInt);
      s:=s2;
      spos:=Pos(sf, s);
    end;}

    //s2:=StringReplace(s, sf, st, [rfReplaceAll, rfIgnoreCase]);
    //s2:=AnsiReplaceStr(s, '<'+sl.Names[i]+'>', sl.ValueFromIndex[i]);
    //s:=s2;
  end;

  //  
  for i:=0 to params.Count-1 do
  begin
    sf:='['+params.Names[i]+']';
    s2:=params.ValueFromIndex[i];
    {$ifdef DELPHI}
    st:=ToAnotherCodePage(s2, 1251, 866);
    {$else}
    st:=UTF8ToCP866(s2);
    {$endif}

    // StringReplace()  !
    MyStringReplace(s, sf, st);
    {len:=Length(sf);
    spos:=Pos(sf, s);
    while spos>0 do
    begin
      s2:=Copy(s, 1, spos-1)+st+Copy(s, spos+len, MaxInt);
      s:=s2;
      spos:=Pos(sf, s);
    end; }

    //s2:=StringReplace(s, sf, st, [rfReplaceAll, rfIgnoreCase]);
    //s2:=AnsiReplaceStr(s, '<'+sl.Names[i]+'>', sl.ValueFromIndex[i]);
    //s:=s2;
  end;


  //  
  //s2:=ToAnotherCodePage(s, 1251, 866);
  Result:=s;
end;

function TLinePrinter.ConvertSimple(text: AnsiString; params: TStrings
  ): AnsiString;
var
  s, s2, sf, st: string;
  ESC, CR, LF, FF, GS, TAB, CAN: string;
  sl: TStringList;
  i: Integer;

begin
  ESC:=Chr($1B);   //
  CR:=Chr($0D);    // Caret Return
  LF:=Chr($0A);    // Line Feed
  FF:=Chr($0C);    //
  GS:=Chr($1D);
  TAB:=Chr($09);   // Horizontal Tab
  CAN:=Chr($18);   // Cancel print
  sl:=TStringList.Create();
  sl.Values['LINE']:='________________________________________________'+LineEnding;
  sl.Values['TAB']:=TAB;
  sl.Values['CR']:='';  // Caret Return
  sl.Values['LF']:=LineEnding;  // Line Feed
  sl.Values['FF']:='===================='+LineEnding;  // Form Feed
  sl.Values['RESET']:='';  // Initialize printer
  // Cut paper fully
  sl.Values['CUT_FULL']:='';
  // Cut paper partially
  sl.Values['CUT_PART']:='';
  // Font 0: 12x24, 1: 9x24, 2: 8x16
  //sl.Values['FONT']:=ESC+'M';
  sl.Values['FONT_0']:='';
  sl.Values['FONT_1']:='';
  sl.Values['FONT_2']:='';
  // 0: USA
  sl.Values['CHARSET']:='';
  // 0: PC437, 7: PC866
  //sl.Values['CODETABLE']:=ESC+'t';
  sl.Values['CODETABLE_DEF']:='';
  sl.Values['CODETABLE_866']:='';
  // HEX 'hv', h-horizontal, v-vertical multiplier
  // $00 - standard, $11 - double, $22 - triple, etc..
  //sl.Values['SIZE']:=GS+'!';
  sl.Values['SIZE_00']:='';
  sl.Values['SIZE_01']:='';
  sl.Values['SIZE_10']:='';
  sl.Values['SIZE_11']:='';
  sl.Values['SIZE_12']:='';
  sl.Values['SIZE_21']:='';
  sl.Values['SIZE_22']:='';
  // 0-none, 1-one dot, 2-two dots
  //sl.Values['UNDERLINE']:=ESC+'-';
  sl.Values['UNDERLINE_0']:='';
  sl.Values['UNDERLINE_1']:='';
  // 0 or 1
  //sl.Values['BOLD']:=ESC+'G';
  sl.Values['BOLD_0']:='';
  sl.Values['BOLD_1']:='';
  // 0 or 1
  //sl.Values['INVERT']:=GS+'B';
  sl.Values['INVERT_0']:='';
  sl.Values['INVERT_1']:='';
  // 0 or 1, for sizes > 1
  //sl.Values['SMOOTH']:=GS+'b';
  sl.Values['SMOOTH_0']:='';
  sl.Values['SMOOTH_1']:='';
  // 0: left, 1: center, 2: right
  //sl.Values['ALIGN']:=GS+'a';
  sl.Values['ALIGN_L']:='';
  sl.Values['ALIGN_C']:='';
  sl.Values['ALIGN_R']:='';
  sl.Values['BEEP']:='';    // Bufferized!
  //sl.Values['TEST']:=GS+'( A pL pH n m';
  // tab positions values, followed by $00
  sl.Values['SET_HT']:='';
  sl.Values['IMAGE_1']:='';
  sl.Values['PAGE_MODE']:='';  // PAGE mode
  sl.Values['STANDARD_MODE']:='';  // STANDARD mode

  s2:=text;
  s:=s2;
  //  
  for i:=0 to sl.Count-1 do
  begin
    sf:='<'+sl.Names[i]+'>';
    st:=sl.ValueFromIndex[i];

    // StringReplace()  !
    MyStringReplace(s, sf, st);
    {len:=Length(sf);
    spos:=Pos(sf, s);
    while spos>0 do
    begin
      s2:=Copy(s, 1, spos-1)+st+Copy(s, spos+len, MaxInt);
      s:=s2;
      spos:=Pos(sf, s);
    end;}

    //s2:=StringReplace(s, sf, st, [rfReplaceAll, rfIgnoreCase]);
    //s2:=AnsiReplaceStr(s, '<'+sl.Names[i]+'>', sl.ValueFromIndex[i]);
    //s:=s2;
  end;

  //  
  for i:=0 to params.Count-1 do
  begin
    sf:='['+params.Names[i]+']';
    s2:=params.ValueFromIndex[i];
    st:=s2;

    // StringReplace()  !
    MyStringReplace(s, sf, st);
    {len:=Length(sf);
    spos:=Pos(sf, s);
    while spos>0 do
    begin
      s2:=Copy(s, 1, spos-1)+st+Copy(s, spos+len, MaxInt);
      s:=s2;
      spos:=Pos(sf, s);
    end; }

    //s2:=StringReplace(s, sf, st, [rfReplaceAll, rfIgnoreCase]);
    //s2:=AnsiReplaceStr(s, '<'+sl.Names[i]+'>', sl.ValueFromIndex[i]);
    //s:=s2;
  end;


  //  
  //s2:=ToAnotherCodePage(s, 1251, 866);
  Result:=s;

end;

function TLinePrinter.LoadTextImage(fname: string): string;
var
  sl: TStringList;
  i, n, bn, ls: Integer;
  mx, my, mxb, xl, xh, yl, yh: Integer;
  s, sr: string;
  bt: Byte;
begin
  Result:='';
  if not FileExists(fname) then Exit;
  sl:=TStringList.Create();
  sl.LoadFromFile(fname);
  // detect max_x and max_y
  mx:=0;
  my:=0;
  for i:=0 to sl.Count-1 do
  begin
    s:=sl[i];
    for n:=1 to Length(s) do
    begin
      if n>mx then mx:=n;
    end;
    Inc(my);
  end;

  mxb:=0;
  sr:='';
  bt:=$00;
  for i:=0 to sl.Count-1 do
  begin
    s:=sl[i];
    bn:=1;
    ls:=Length(s);
    for n:=1 to mx do
    begin
      bt:=bt shl 1;
      // test for spaces inside string
      if (n<=ls) and (s[n]=' ') then
      begin
        //bt:=bt or $80; // 1000 0000 b
        bt:=bt or $01; // 0000 0001 b
      end;
      Inc(bn);
      if bn>8 then
      begin
        bn:=1;
        sr:=sr+Chr(bt);
        bt:=$00;
      end;
    end;
    // Adding remaining bits
    if bn>1 then
    begin
      sr:=sr+Chr(bt);
      bt:=$00;
    end;
    if mxb=0 then mxb:=Length(sr);
  end;

  sl.Free();

  // <img_prefix><mode><xL_xH><yL_yH><>
  Result:=#$1D+#$76+#$30+#$03; // prefix + mode
  xl:=mxb;
  xh:=0;
  if mxb>255 then
  begin
    xh:=mxb div 256;
    xl:=mxb mod 256;
  end;
  yl:=my;
  yh:=0;
  if my>255 then
  begin
    yh:=my div 256;
    yl:=my mod 256;
  end;
  Result:=Result+Chr(xl)+Chr(xh)+Chr(yl)+Chr(yh)+sr;
end;

function TLinePrinter.LoadTemplate(FileName: string; SectionName: string
  ): Boolean;
var
  sl: TStringList;
  i: Integer;
begin
  Result:=False;
  if not FileExists(FileName) then Exit;
  sl:=TStringList.Create();
  sl.LoadFromFile(FileName);
  Template:='';
  for i:=0 to sl.Count-1 do
  begin
    // ignore comments
    if Pos('#', sl[i])=1 then Continue;
    Template:=Template+sl[i];
  end;
  sl.Free();
  if SectionName<>'' then
  begin
    Sections.Values[SectionName]:=Template;
    Template:='';
  end;
  Result:=True;
end;

procedure TLinePrinter.ClearDoc();
begin
  RawDocument:='';
end;

procedure TLinePrinter.AddSection(ASectionName: string; AParams: TStrings);
var
  s: AnsiString;
begin
  // convert previous section
  if SimpleMode then s:=ConvertSimple(Template, Params) else s:=Convert(Template, Params);
  RawDocument:=RawDocument+s;

  // add new section
  if ASectionName<>'' then
  begin
    Template:=Sections.Values[ASectionName];

    if Assigned(AParams) then
    begin
      Params.Clear();
      Params.Assign(AParams);
    end;

    if SimpleMode then s:=ConvertSimple(Template, Params) else s:=Convert(Template, Params);
    RawDocument:=RawDocument+s;

    Template:='';
  end;

end;

function TLinePrinter.Print(): Boolean;
var
  s: string;
begin
  Result:=False;
  if not Assigned(DataPort) then Exit;
  if not DataPort.Active then
  begin
    DataPort.Active:=True;
    Sleep(200);
  end;
  Result:=DataPort.Push(RawDocument);
end;

function TLinePrinter.PrintToFile(AFileName: string): Boolean;
var
  sl: TStringList;
  s: string;
begin
  Result:=False;
  if AFileName='' then Exit;
  // check directory
  s:=ExtractFileDir(AFileName);
  if (s<>'') and not DirectoryExists(s) then CreateDir(s);
  sl:=TStringList.Create();
  sl.Text:=RawDocument;
  try
    sl.SaveToFile(AFileName);
    Result:=True;
  finally
    sl.Free();
  end;
end;

procedure TLinePrinter.SetParam(AName, AValue: string);
begin
  Params.Values[AName]:=AValue;
end;

procedure TLinePrinter.OnDataAppearHandler(Sender: TObject);
var
  s: string;
  sStatus: string;
begin
  if DataPort.PeekSize < 3 then Exit;
  s:=DataPort.Pull(2); // Skip 2 status bytes
  s:=DataPort.Pull();
  if FStatusMode = $01 then FPrinterStatus:=Ord(s[1])
  else if FStatusMode = $02 then FOfflineStatus:=Ord(s[1])
  else if FStatusMode = $03 then FErrorStatus:=Ord(s[1]);

  if (FStatusMode > 0) and ((FPrinterStatus and 8) > 0) and (FOfflineStatus = 0) then // offline status
  begin
    CheckStatus($02);
    Exit;
  end;

  if (FStatusMode > 0) and ((FOfflineStatus and 64) > 0) and (FErrorStatus = 0) then // error status when offline
  begin
    CheckStatus($03);
    Exit;
  end;

  if Assigned(FOnStatus) then
  begin
    if (FStatusMode > 0) then
    begin
      // Decode status
      FReadyToPrint:=False;
      sStatus:='';
      if (FPrinterStatus and 8) > 0 then sStatus:=sStatus+'Offline; ';
      if (FPrinterStatus and 32) > 0 then sStatus:=sStatus+'Waiting recovery; ';
      if (FPrinterStatus and 64) > 0 then sStatus:=sStatus+'FEED pressed; ';

      if (FOfflineStatus and 4) > 0 then sStatus:=sStatus+'Cover open; ';
      if (FOfflineStatus and 8) > 0 then sStatus:=sStatus+'Paper feed; ';
      if (FOfflineStatus and 32) > 0 then sStatus:=sStatus+'Paper out; ';
      if (FOfflineStatus and 64) > 0 then sStatus:=sStatus+'Error detected; ';

      if (FErrorStatus and 4) > 0 then sStatus:=sStatus+'B.M. detection error; ';
      if (FErrorStatus and 8) > 0 then sStatus:=sStatus+'Auto cutter error; ';
      if (FErrorStatus and 32) > 0 then sStatus:=sStatus+'Unrecoverable error; ';
      if (FErrorStatus and 64) > 0 then sStatus:=sStatus+'Auto recovery error; ';

      if sStatus='' then
      begin
        sStatus:='OK';
        FReadyToPrint:=True;
      end;
      if sStatus <> FLastStatusStr then
      begin
        FLastStatusStr:=sStatus;
        FOnStatus(sStatus);
      end;
      FStatusMode:=$00;
    end
    else
    begin
      if s[1]='_' then FOnStatus(s)
      else FOnStatus(StrToHex(s));
    end;
  end;

  if (FStatusMode > 0) then FStatusMode:=$00;
end;

procedure TLinePrinter.FSetDataPort(Value: TDataPort);
begin
  if Value = FDataPort then Exit;
  // Disconnect previous DataPort
  if Assigned(FDataPort) then
  begin
    FDataPort.OnDataAppear:=nil;
    FDataPort.OnError:=nil;
    FDataPort.Active:=False;
  end;
  // Assign new DataPort
  FDataPort:=Value;
  if Assigned(FDataPort) then
  begin
    FDataPort.OnDataAppear:=@OnDataAppearHandler;
  end;
end;

function TLinePrinter.LoadBitmapImage(fname: string): string;
var
  pic: TPicture;
  //bmp: TBitmap;
  can: TCanvas;
  x, y: Integer;
begin
  Result:='';
  Exit;
  pic:=TPicture.Create();
  pic.LoadFromFile(fname);
  //bmp:=pic.Bitmap;
  can:=pic.Bitmap.Canvas;
  for y:=can.ClipRect.Top to can.ClipRect.Bottom do
  begin
    for x:=can.ClipRect.Left to can.ClipRect.Right do
    begin

    end;
  end;
  pic.Free();
end;

function TLinePrinter.ReadyToPrint(): Boolean;
begin
  Result:=FReadyToPrint;
end;

procedure TLinePrinter.OnStatusTimerHandler(Sender: TObject);
begin
  self.CheckStatus();
end;

end.
